#! /usr/local/bin/wish -f

# server machine name
# the machine name has to match the redering platform's name exactly.
set server nova.npac.syr.edu

#Initialize url stuff
set urllistfile "loclist"

# Initialize variables
set minzoom 0
set maxzoom 3
set zoom_value 0

set image_llx 0
set image_lly 0
set image_urx 0
set image_ury 0

set basephotoname ./test
set photoname ./test_0.1.1.ppm

set x_max 640
set y_max 480

#Initialize the four corners of the screen, assuming a zoom level of
#0 and that the top lefthand corner of the image is being shown
set upper_left_x 0 
set upper_left_y $y_max
set upper_right_x 300 
set upper_right_y $y_max
set lower_left_x 0 
set lower_left_y [expr $y_max - 300]
set lower_right_x 300 
set lower_right_y [expr $y_max - 300]

#initialize the variables that return the answer to the queryxy
#procedure 
set current_screen_x -1
set current_screen_y -1

bind Viewer <Enter> {colorfocus %W}
bind Photo <Enter> {colorfocus %W}
bind Toplevel <Enter> {colorfocus %W}

if {[file exists [format "%s_%d.1.2.ppm" $basephotoname 0]]} {
    set image_height_index 1
    set image_width_index 1
} else {
    set image_height_index 0
    set image_width_index 0
}


wm title . "Terrain interface demo"
wm minsize . 1 1

frame .screen
pack .screen -side top -fill both

frame .screen.f1
pack .screen.f1 -side top -fill both

# Update the image when the user changes the zoom value 
proc morezoom value {
    global photo_item
    global photoname
    global basephotoname
    global image_llx
    global image_lly
    global image_urx
    global image_ury
    global image_height_index
    global image_width_index
    global zoom_value
    global top_left_x
    global top_left_y
    global top_right_x
    global top_right_y
    global bottom_left_x
    global bottom_left_y
    global bottom_right_x
    global bottom_right_y
    global y_max

    set $value [.control.zoom.vscale get]
    set photoname [format "%s_%d.1.1.ppm" $basephotoname $value]
    set photoname2 [format "%s_%d.1.2.ppm" $basephotoname $value]
    if {[file exists $photoname2]} {
	set photoname3 [format "%s_%d.2.1.ppm" $basephotoname $value]
	set photoname4 [format "%s_%d.2.2.ppm" $basephotoname $value]
	set image_height_index 1
	set image_width_index 1
    } else {
	set image_height_index 0
	set image_width_index 0
    }

    .control.map.canvas create rectangle $image_llx $image_lly $image_urx \
	    $image_ury -outline bisque -fill bisque
    if {[file exists $photoname2]} {
	find_multi_image_dims $photoname $photoname2 $photoname3 $photoname4 
    } else {
	find_image_dims $photoname
    }

    set image_dim [format "%f %f %f %f" $image_llx $image_lly \
	    $image_urx $image_ury]

    .control.map.canvas configure -scrollregion $image_dim
    
    set photo_item .control.map.canvas-[.control.map.canvas create photo \
	    $image_llx $image_lly $image_urx \
	    $image_ury]

    if {[file exists $photoname2]} {
        catppm $photo_item $photoname $photoname2 $photoname3 $photoname4
    } else {
        readppm $photo_item $photoname
    }

    set top_left_x 0 
    set top_left_y $y_max
    set top_right_x [expr 300/[expr pow(2,$value)]]
    set top_right_y $y_max
    set bottom_left_x 0 
    set bottom_left_y [expr $y_max - 300/[expr pow(2,$value)]] 
    set bottom_right_x $top_right_x 
    set bottom_right_y $bottom_left_y
    set zoom_value $value
}


frame .control
pack .control -side top -fill both

frame .control.zoom
label .control.zoom.label -text "Zoom"
scale .control.zoom.vscale -orient vertical \
      -length 180 -from $maxzoom -to $minzoom \
      -tickinterval 1 -bg Bisque1 -command morezoom

pack .control.zoom.label .control.zoom.vscale \
     -in .control.zoom -side top

#determine the height and width of the image in value (value must be
#the name of a ppm file), and set image_urx and image_ury accordingly
proc find_image_dims value {
    global image_urx
    global image_ury

    set nums ""
    set f [open $value r]
    gets $f nums
    gets $f nums
    scan $nums "%d %d" image_urx image_ury
}

#determine the height and width of the image that would be produced by
#merging the 4 ppm files file1 - file4; set image_urx and image_ury
#accordingly 
proc find_multi_image_dims {file1 file2 file3 file4} {
    global image_urx
    global image_ury

    set nums ""
    set tempheight 0
    set tempwidth 0
    set image_urx 0
    set image_ury 0
    foreach file [list $file1 $file2] {
	set f [open $file r]
	gets $f nums
	gets $f nums
	scan $nums "%d %d" tempwidth tempheight
	set image_urx [expr $image_urx + $tempwidth]
    }
    foreach file [list $file1 $file3] {
	set f [open $file r]
	gets $f nums
	gets $f nums
	scan $nums "%d %d" tempwidth tempheight
	set image_ury [expr $image_ury + $tempheight]
    }
}

frame .control.map

find_image_dims $photoname
set image_dim [format "%f %f %f %f" $image_llx $image_lly $image_urx $image_ury]

canvas .control.map.canvas -width 300 -height 300 \
       -xscrollcommand ".control.map.scrollx set" \
       -yscrollcommand ".control.map.scrolly set" \
       -scrollregion $image_dim

set photo_item .control.map.canvas-[.control.map.canvas create photo \
                                    $image_llx $image_lly $image_urx $image_ury]
#readppm $photo_item $photoname
morezoom 0


#ANI

#create a scrolled canvas for the url list
frame .list
pack .list -side left -fill both

set frame_llx 0
set frame_lly 0
set frame_urx 100
set frame_ury 300
set frame_dim [format "%f %f %f %f" $frame_llx $frame_lly $frame_urx $frame_ury]
canvas .list.canvas -width 100 -height 300 \
       -yscroll {.list.scrolly set} \
       -scrollregion $frame_dim
scrollbar .list.scrolly -orient vertical \
       -command {list_scrolly}
pack .list.scrolly -side right -fill y
pack .list.canvas -side left -fill both

#get the url list
set fd [open $urllistfile r]
gets $fd nurl
for {set i 0} {$i < $nurl} {incr i 1} {
  gets $fd temp
  scan $temp "%d %d %s %s" northing($i) easting($i) loc_name($i) url($i)
}
close $fd

#DEV - put check for map extent here to find buttons to be displayed on list

#create text and dot widgets for the url list
for {set i 0} {$i < $nurl} {incr i 1} {
    set texttag($i) [.list.canvas create text 0.15c $i.75c -text \
	    $loc_name($i) -fill blue -anchor sw]
#DEV - this is where the screen x, y for the circle has be computed
     set loctag($i) [.control.map.canvas create oval 30 50 40 60 -fill red]
     set locttag($i) [.control.map.canvas create text 25 45 -text \
	     loc_name($i) -fill red -anchor sw]

}

#and bind them
for {set i 0} {$i < $nurl} {incr i 1} {
    .list.canvas bind $texttag($i) <Button-1> "DisplayText $i 30 50 40 60"
    .control.map.canvas bind $loctag($i) <Button-1> "SendUrl $i url"
    .control.map.canvas bind $locttag($i) <Button-1> "SendUrl $i url"
    .control.map.canvas bind $loctag($i) <Button-3> "DeleteTag $i"
    .control.map.canvas bind $locttag($i) <Button-3> "DeleteTag $i"
}

#create a canvas for the display and quit buttons
frame .display
pack .display -side bottom -fill y

button .display.quit -text "Quit" -command Quit
button .display.wipe -text "Wipe" -command Wipe
button .display.redisplay -text "Redisplay" -command Redisplay
pack .display.quit -in .display -side left -padx 10
pack .display.wipe -in .display -side left -padx 10
pack .display.redisplay -in .display -side left -padx 10


proc Wipe {} {

    global loctag
    global locttag
    global nurl
    
    for {set i 0} {$i < $nurl} {incr i 1} {
	.control.map.canvas delete $loctag($i)
	.control.map.canvas delete $locttag($i)
    }
}

proc Redisplay {} {

    global loctag
    global locttag
    global nurl
    global loc_name
    
    for {set i 0} {$i < $nurl} {incr i 1} {
	set loctag($i) [.control.map.canvas create oval 30 50 40 60 -fill red]
	set locttag($i) [.control.map.canvas create text 25 45 -text \
		loc_name($i) -fill red -anchor sw]
	.control.map.canvas bind $loctag($i) <Button-1> "SendUrl $i url"
	.control.map.canvas bind $locttag($i) <Button-1> "SendUrl $i url"
	.control.map.canvas bind $loctag($i) <Button-3> "DeleteTag $i"
	.control.map.canvas bind $locttag($i) <Button-3> "DeleteTag $i"
    }
}

proc Quit {} {

   quit
   exit
}

proc DisplayText {i x1 y1 x2 y2} {
  
    global loctag 
    
    set loctag($i) [.control.map.canvas create oval $x1 $y1 $x2 $y2 \
	    -fill red]
    set locttag($i) [.control.map.canvas create text 25 45 -text \
		loc_name($i) -fill red -anchor sw]
    .control.map.canvas bind $loctag($i) <Button-1> "SendUrl $i url"
    .control.map.canvas bind $locttag($i) <Button-1> "SendUrl $i url"
    .control.map.canvas bind $loctag($i) <Button-3> "DeleteTag $i"
    .control.map.canvas bind $locttag($i) <Button-3> "DeleteTag $i"

}

proc SendUrl {i urln} {

    upvar $urln url
    sendurl $url($i)
}

proc DeleteTag {i} {

    global loctag
    global locttag

    .control.map.canvas delete $loctag($i)
    .control.map.canvas delete $locttag($i)
    set $loctag($i) 0
}

#ANI

proc put_corners {} {
    global lower_left_x
    global lower_left_y
    global lower_right_x
    global lower_right_y
    global upper_left_x
    global upper_left_y
    global upper_right_x
    global upper_right_y

    puts $upper_left_x
    puts $upper_left_y
    puts $upper_right_x
    puts $upper_right_y
    puts $lower_left_x
    puts $lower_left_y
    puts $lower_right_x
    puts $lower_right_y
    puts ""
}

#replace the image currently showing in the window with the
#concatenation of the 4 images in the ppm files file1 - file4
proc update_image {file1 file2 file3 file4} {
    global .control.map.canvas
    global image_dim
    global image_llx
    global image_lly
    global image_urx
    global image_ury
    global photo_item

    .control.map.canvas create rectangle 0 0 300 300 \
	    -outline bisque -fill bisque
    find_multi_image_dims $file1 $file2 $file3 $file4
    set image_dim [format "%f %f %f %f" $image_llx $image_lly \
	    $image_urx $image_ury]
    
    .control.map.canvas configure -scrollregion $image_dim
    set photo_item .control.map.canvas-[.control.map.canvas create photo \
	    $image_llx $image_lly $image_urx \
	    $image_ury]
     catppm $photo_item $file1 $file2 $file3 $file4
}

#function called when the user manipulates the horizontal scrollbar;
#updates the value of the scrollbar, determines whether or not a new
#block of image must be read, and acts accordingly 
proc scrollx_update value {
    global .control.map.scrollx
    global .control.map.canvas
    global basephotoname
    global photoname
    global zoom_value
    global image_height_index
    global image_width_index
    global photo_item
    global lower_left_x
    global lower_right_x
    global upper_left_x
    global upper_right_x
    global x_max
    global y_max

    set coords 0
    set c1 0
    set c2 0
    set c3 0
    set c4 0
    set p1 ""
    set p2 ""
    set p3 ""
    set p4 ""
    .control.map.canvas xview $value
    set coords [.control.map.scrollx get]
    scan $coords "%d %d %d %d" c1 c2 c3 c4
    if {[expr $value >= 0]} {
	set c3 $value
    } else {
	set c3 0
    }
    if {[expr [expr $c3 + $c2] <= $c1]} {
	set c4 [expr $c3 + $c2]
    } else {
	set c4 $c1
    }
    if {[expr $c2 == [expr $c4 - $c3]]} {
	set temp [expr $c3*10/[expr pow(2,$zoom_value)]]
    } else {
	set temp [expr [expr [expr $c4-$c2]*10]/[expr pow(2,$zoom_value)]]
    }
    set lower_left_x [expr [expr $image_width_index-1]*300/[expr pow(2,$zoom_value)] + $temp]
    set lower_right_x [expr $lower_left_x + 300/[expr pow(2,$zoom_value)]]
    set upper_left_x $lower_left_x
    set upper_right_x $lower_right_x
    if {[expr [expr $c1 == $c4] || [expr $c1 == [expr $c4 + 1]] && [expr $image_height_index != 0]]} {
	set p1 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		$image_height_index [expr $image_width_index + 1]]
	set p2 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		$image_height_index [expr $image_width_index + 2]]
	set p3 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index + 1] [expr $image_width_index + 1]]
	set p4 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index + 1] [expr $image_width_index + 2]]
	if {[file exists $p1] && [file exists $p2] && [file exists $p3] && \
		[file exists $p4]} {
	    update_image $p1 $p2 $p3 $p4
	    .control.map.canvas xview 0
	}
    }
    
    if {[expr [expr $c3 == 0] && [expr $image_height_index != 0]]} {
	set p1 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		$image_height_index [expr $image_width_index - 1]]
	set p2 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		$image_height_index $image_width_index]
	set p3 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index + 1] [expr $image_width_index - 1]]
	set p4 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index + 1] $image_width_index]
	if {[file exists $p1] && [file exists $p2] && [file exists $p3] && \
		[file exists $p4]} {
	    update_image $p1 $p2 $p3 $p4
	    set coords [.control.map.scrollx get]
	    scan $coords "%d %d %d %d" c1 c2 c3 c4
	    .control.map.canvas xview [expr $c2 - 1]
	    set temp [expr 20/[expr pow(2,$zoom_value)]]
	    set upper_left_x [expr $upper_left_x - $temp]
	    set upper_right_x [expr $upper_right_x - $temp]
	    set lower_left_x [expr $lower_left_x - $temp]
	    set lower_right_x [expr $lower_right_x - $temp]
	    set image_width_index [expr $image_width_index - 1]
    }   
}
}

#function called when the user manipulates the vertical scrollbar;
#updates the value of the scrollbar, determines whether or not a new
#block of image must be read, and acts accordingly 
proc scrolly_update value {
    global .control.map.scrolly
    global .control.map.canvas
    global basephotoname
    global photoname
    global photo_item
    global zoom_value
    global image_height_index
    global image_width_index
    global lower_left_y
    global lower_right_y
    global upper_left_y
    global upper_right_y    
    global x_max
    global y_max
    
    set coords 0
    set c1 0
    set c2 0
    set c3 0
    set c4 0
    set p1 ""
    set p2 ""
    set p3 ""
    set p4 ""
    .control.map.canvas yview $value
    set coords [.control.map.scrolly get]
    scan $coords "%d %d %d %d" c1 c2 c3 c4
    if {[expr $value >= 0]} {
	set c3 $value
    } else {
	set c3 0
    }
    if {[expr [expr $c3 + $c2] <= $c1]} {
	set c4 [expr $c3 + $c2]
    } else {
	set c4 $c1
    }
    puts $c1
    puts $c2
    puts $c3
    puts $c4
    if {[expr $c2 == [expr $c4 - $c3]]} {
	set temp [expr $c3*10/[expr pow(2,$zoom_value)]]
    } else {
	set temp [expr [expr [expr $c4-$c2]*10]/[expr pow(2,$zoom_value)]]
    }
    set lower_left_y [expr $y_max - [expr $image_height_index*300/[expr pow(2,$zoom_value)] + $temp]]
    set lower_right_y $lower_left_y
    set upper_left_y [expr $lower_left_y + 300/[expr pow(2,$zoom_value)]]
    set upper_right_y $upper_left_y
    if {[expr [expr $c1 == $c4] || [expr $c1 == $c4 + 1]] && [expr $image_height_index != 0]} {
	set p1 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index + 1] $image_width_index]
	set p2 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index + 1] [expr $image_width_index + 1]]
	set p3 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index + 2] $image_width_index]
	set p4 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index + 2] [expr $image_width_index + 1]]
	if {[file exists $p1] && [file exists $p2] && [file exists $p3] && \
		[file exists $p4]} {
	    update_image $p1 $p2 $p3 $p4
	    .control.map.canvas yview 0
            if {[expr $c1 == $c4]} {
		set temp [expr 10/[expr pow(2,$zoom_value)]]
	    } else {
		set temp [expr 20/[expr pow(2,$zoom_value)]]
	    }
	    set lower_left_y [expr $lower_left_y - $temp]
	    set lower_right_y [expr $lower_right_y - $temp]
	    set upper_left_y [expr $upper_left_y - $temp]
	    set upper_right_y [expr $upper_right_y - $temp]
	    set image_height_index [expr $image_height_index + 1]
	}
    }

    if {[expr [expr $c3 == 0] && [expr $image_height_index != 0]]} {
	set p1 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index - 1] $image_width_index]
	set p2 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		[expr $image_height_index - 1] [expr $image_width_index + 1]]
	set p3 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		$image_height_index $image_width_index]
	set p4 [format "%s_%d.%d.%d.ppm" $basephotoname $zoom_value \
		$image_height_index [expr $image_width_index + 1]]
	if {[file exists $p1] && [file exists $p2] && [file exists $p3] && \
		[file exists $p4]} {
	    update_image $p1 $p2 $p3 $p4
	    set coords [.control.map.scrolly get]
	    scan $coords "%d %d %d %d" c1 c2 c3 c4
	    .control.map.canvas yview [expr $c2 - 1]
	    set temp [expr 20/[expr pow(2,$zoom_value)]]
	    set lower_left_y [expr $lower_left_y + $temp]
	    set lower_right_y [expr $lower_right_y + $temp]
	    set upper_left_y [expr $upper_left_y + $temp]
	    set upper_right_y [expr $upper_right_y + $temp]
	    set image_height_index [expr $image_height_index - 1]
	}
    }
}

scrollbar .control.map.scrollx -orient horiz \
	-command {scrollx_update} \
	-bg maroon
pack .control.map.scrollx -side bottom -fill x

scrollbar .control.map.scrolly -orient vert \
	-command {scrolly_update} \
	-bg maroon 
pack .control.map.scrolly -side right -fill y

pack .control.map.scrollx .control.map.scrolly \
.control.map.canvas -in .control.map

pack .control.map .control.zoom -in .control -side right

pack .screen .control -in . -side top

#procedure that, given an x and y UTM coordinate, determines the
#screen coordinates of that point; updates the global variables
#current_screen_x and current_screen_y with the answer
#Any point between (0,0) and (300,300) is a valid screen coordinate,
#any point not between those 2 points is not on the screen 
proc queryxy{x y} {
    global lower_left_x
    global lower_left_y
    global current_screen_x
    global current_screen_y
    global zoom_value

    set temp [expr pow(2,$zoom_value)]
    set current_screen_x [expr [expr $lower_left_x - $x]/$temp]
    set current_screen_y [expr [expr $lower_left_y - $y]/$temp]
}
