#include <stdio.h>
#ifndef NO_STDLIB_H
#include <stdlib.h>
#else
char *getenv();
#endif
#include "inet.h"
#include "shmem.h"
#include "start_client.h"

#define MAX_ENTRIES 10000

#define WESTING 100000     /* west edge of map in UTM coordinates */
#define NORTHING 100000    /* north edge of map in UTM coordiantes */
#define RESOLUTION 30      /* 30m resolution data */
#define NTAGS      10      /* number of tags on map */

typedef struct {
    char *name;
    char *val;
} entry;

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
char x2c(char *what);
void unescape_url(char *url);
void plustospace(char *str);

void Environ(void);

int semid = 0, shmid = 0;   /* global for signal handler */

main(int argc, char *argv[]) {

    entry entries[MAX_ENTRIES];
    register int x,m=0;
    int cl;
    char hostname[MAXHOSTNAMELEN+1];
    char combuf[320], tempbuf[80];
    char errmsg[320];
    char url[160];
    int  win_id, init, ret;
    FILE *fd = NULL, *logfile = NULL;
    int  count = 0;
    Point *mouse_click;
    Tag   **tag_list;
    int   i, ntags, flag = OFF, status;
    int  fifo_id;

    printf("Content-type: text/plain%c%c",10,10);

    cl = atoi(getenv("CONTENT_LENGTH"));

    for(x=0;cl && (!feof(stdin));x++) {
        m=x;
        entries[x].val = fmakeword(stdin,'&',&cl);
        plustospace(entries[x].val);
        unescape_url(entries[x].val);
        entries[x].name = makeword(entries[x].val,'=');
    }

    logfile = fopen("log", "w");
/* debug */
    fprintf(logfile, "%s=%s\n", entries[0].name, entries[0].val);
    fflush(logfile);
    fprintf(logfile, "%s\n", "starting");
    fflush(logfile); 

    if (!fork()) { /* child runs client program */
      strcpy(combuf, "./concat mydemo.tcl");
      system(combuf);
      _exit(0);
    }

    /* parent runs the gateway prcess */
    /* create the FIFO to write the XDisplay to client process */
    open("/tmp/fifo.disp", O_CREAT | O_EXCL, 066); /* first, check if the
						      file already exists */
    unlink("/tmp/fifo.disp");  /* and then close it */
    if (mkfifo("/tmp/fifo.disp", O_WRONLY | 0666) < 0) {
      strcpy(errmsg, "ERROR: mkfifo failed");
      fprintf(logfile, "%s errno = %d\n", errmsg, errno);
      fflush(logfile);
      exit(1);
    }
    fifo_id = open("/tmp/fifo.disp", O_WRONLY);
    write(fifo_id, entries[0].val, MAXHOSTNAMELEN);

    /* create the semaphores for synchronization with client process */
    if ((semid = SemCreate2(SEMKEY_CGI, 0, 1)) == ERROR) {
      fprintf(logfile, "%s", "ERROR in creating semaphore \n");
      fflush(logfile);
      flag = ON;
      goto cleanup;
    }
    CatchSignals(); /* establish the signal handler to remove ipc variables on
                       abnormal termination */
    fprintf(logfile, "semid = %d\n", semid);
    fflush(logfile);

    /* create the shared memory segment for the mouse click position */
/*    if ((shmid = ShmAlloc(SHMKEY, sizeof(Point), mouse_click)) == ERROR) {
      fprintf(logfile, "%s", "ERROR in creating shared memory \n");
      fflush(logfile);
      flag = ON;
      goto cleanup;
    } */
    if ((shmid = ShmAlloc(SHMKEY, sizeof(Point))) == ERROR) {
      fprintf(logfile, "%s", "ERROR in creating shared memory \n");
      fflush(logfile);
      flag = ON;
      goto cleanup;
    }
    if ((mouse_click = (Point *) shmat(shmid, (char *)0, 0)) 
                     == (Point *)ERROR) {
      strcpy(errmsg, "ERROR: can't attach shared memory");
      perror(errmsg);
      flag = ON;
      goto cleanup;
    }
    fprintf(logfile, "shmid = %d\n", shmid);
    fflush(logfile);

    strcpy(hostname, entries[0].val);
    fprintf(logfile, "hostname = %s\n", hostname);
    fflush(logfile);

    fd = fopen("loclist", "r");
    fscanf(fd, "%d", &ntags);
    tag_list = (Tag **)malloc(ntags * sizeof(Tag *));
    for (i = 0; i < ntags; i++)
      tag_list[i] = (Tag *)malloc(sizeof(Tag));
    GetTagList(fd, ntags, tag_list);
    for (i = 0; i < ntags; i++) {
      fprintf(logfile, "%d %d %s %s \n", tag_list[i]->easting, 
	      tag_list[i]->northing, tag_list[i]->loc_name, tag_list[i]->url);
      fflush(logfile);
    }

    while (TRUE) {
      SemCliWait(semid);  /* wait for client to write something */
      count++;
      if (mouse_click->row != END) {
	    OpenUrl(hostname, mouse_click->url, logfile);
      } else  /* end */
	    break;
      SemCgiSignal(semid);
    }

 cleanup:
    fprintf(logfile, "%s\n", "exiting!");
    fflush(logfile);
    /* remove all ipc variables */
    wait(NULL);     /* wait for child to exit */
    if (flag) KillUI();
    unlink("/tmp/fifo.disp");
    if (semid) SemRem(semid);
    if (shmid) ShmRel(shmid);
    /* close all io streams */
    if (fd) fclose (fd);
    if (logfile) fclose (logfile);

    exit(OK);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int ParseWinInfo(char *filename) {

  char buf[100];
  FILE *fd;
  int  d;
  
  fd = fopen(filename, "r");
  fscanf(fd, "%s %s %s %d", buf, buf, buf, &d);
  fclose (fd);

  return (d);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
void MyInterr(int signo) {

  if (semid) SemRem(semid);
  if (shmid) ShmRel(shmid);
  _exit(0);

}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
void CatchSignals() {

  signal(SIGINT, MyInterr);
  signal(SIGSEGV, MyInterr); signal(SIGSTOP, MyInterr);
  signal(SIGQUIT, MyInterr); signal(SIGKILL, MyInterr);
  signal(SIGHUP, MyInterr); signal(SIGTSTP, MyInterr);
  signal(SIGABRT, MyInterr); signal(SIGTERM, MyInterr);
  signal(SIGUSR1, MyInterr); signal(SIGUSR2, MyInterr);

}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int GetUrl(Point *p, int n, Tag **tag_list, char *url, FILE *logfile) {

  int utm_east, utm_south;
  int i;

  utm_east = WESTING + (p->column * RESOLUTION);
  utm_south = NORTHING + (p->row * RESOLUTION);

  for (i = 0; i < n; i++) {
    if ((tag_list[i]->easting == utm_east) &&
	(tag_list[i]->northing == utm_south)) {
      strcpy(url, tag_list[i]->url);
      return (OK);
    }
  }
  
  fprintf(logfile, "%s\n", "ERROR: could not locate URL for location");
  fflush(logfile);

  return (ERROR);
  
}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
void GetTagList(FILE *fd, int n, Tag **tag_list) {

  int  i;
  Tag  *ptr;
  
  for (i = 0; i < n; i++) {
    fscanf(fd, "%d %d %s %s", &(tag_list[i]->easting), 
	   &(tag_list[i]->northing), tag_list[i]->loc_name,
	   tag_list[i]->url);
  }
  
  return;
}



/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
void OpenUrl(char *hostname, char *url, FILE *logfile) {

  char combuf[320];

  strcpy(combuf, "netscape -display ");
  strcat(combuf, hostname);
  strcat(combuf, " -remote 'openURL(");
  strcat(combuf, url);
  strcat(combuf, ")'");
  system(combuf);
  fprintf(logfile, "%s\n", combuf);
  fflush(logfile);

}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
void KillUI() {
  
  FILE *fd;
  int  pid = 0;

  while (TRUE) {
    fd = fopen("../../ui/ui_pid", "r");
    fscanf(fd, "%d", &pid);
    if (pid) break;
    fclose (fd);
  }
	 
  if(fd) fclose(fd);
  kill(pid, SIGUSR1);

  return;
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: Using putenv to stick to upcoming POSIX.1 standard
----------------------------------------------------------------------- */
void Environ() {

  char str[80];
  char hostname[MAXHOSTNAMELEN+1];

  FingerHost(hostname, MAXHOSTNAMELEN+1);
  strcpy(str, "DISPLAY=");
  strcat(str, hostname);
  strcat(str, ":0");
  putenv(str);

  return;
}
